"""
app config
"""

from django.apps import AppConfig

from aa_rss_to_discord import __version__


class AaRssToDiscordConfig(AppConfig):
    """
    application config
    """

    name = "aa_rss_to_discord"
    label = "aa_rss_to_discord"
    verbose_name = "RSS to Discord v{}".format(__version__)



"""
constants
"""

from django.utils.text import slugify

from aa_rss_to_discord import __version__

VERBOSE_NAME = (
    "AA RSS To Discord - Alliance Auth "
    "module to post news from RSS feeds to your Discord"
)
USERAGENT = "{verbose_name} v{version} {github_url}".format(
    verbose_name=slugify(VERBOSE_NAME, allow_unicode=True),
    version=__version__,
    github_url="https://github.com/ppfeufer/aa-rss-to-discord",
)



"""
Our Models
"""

from aadiscordbot.models import Channels

from django.db import models
from django.utils.translation import gettext as _


class RssFeeds(models.Model):
    """
    Model :: RssFeeds
    """

    name = models.CharField(max_length=254, default="")
    url = models.CharField(max_length=254, default="")
    discord_channel = models.ForeignKey(
        Channels,
        related_name="+",
        null=True,
        blank=True,
        default=None,
        on_delete=models.SET_NULL,
    )

    class Meta:
        """
        Meta definitions
        """

        default_permissions = ()
        verbose_name = _("RSS Feed")
        verbose_name_plural = _("RSS Feeds")

    def __str__(self):
        return f'RSS Feed "{self.name}" for channel {self.discord_channel}'



"""
Django admin declarations
"""

from django.contrib import admin

from aa_rss_to_discord.models import RssFeeds


def custom_filter(title):
    """
    custom filter for model properties
    :param title:
    :return:
    """

    class Wrapper(admin.FieldListFilter):
        """
        custom_filter :: wrapper
        """

        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title

            return instance

    return Wrapper


@admin.register(RssFeeds)
class RssFeedsAdmin(admin.ModelAdmin):
    """
    RssFeedsAdmin
    """

    list_display = ("name", "url", "discord_channel")
    ordering = ("name",)

    list_filter = ("name", "url", "discord_channel")

    search_fields = ("name", "url", "discord_channel")



"""
AA RSS To Discord Tasks
"""

import logging

import feedparser
from celery import shared_task

from django.conf import settings
from django.core.cache import cache
from django.utils.text import slugify

from allianceauth.services.hooks import get_extension_logger
from allianceauth.services.tasks import QueueOnce

from aa_rss_to_discord import __title__
from aa_rss_to_discord.models import RssFeeds
from aa_rss_to_discord.utils import LoggerAddTag

logger = LoggerAddTag(get_extension_logger(__name__), __title__)

# poll every x seconds
RSS_POLL_TIME = 300


@shared_task(**{"base": QueueOnce})
def fetch_rss() -> None:
    """
    fetch RSS feeds and post to Discord
    :return:
    :rtype:
    """

    rss_feeds = RssFeeds.objects.all()

    if "aadiscordbot" in settings.INSTALLED_APPS and rss_feeds:
        import aadiscordbot.tasks

        for rss_feed in rss_feeds:
            try:
                logger.info(f"Fetching RSS Feed {rss_feed.name}")

                rss_cache_key = slugify(rss_feed.url, allow_unicode=True)

                feed = feedparser.parse(rss_feed.url)
                latest_entry = feed.entries[0]
                feed_entry_guid = latest_entry.guid

                if (
                    cache.get(rss_cache_key + "_feed_entry_guid") is None
                    or cache.get(rss_cache_key + "_feed_entry_guid") != feed_entry_guid
                ):
                    post_entry = True

                    # Filter might come later ...
                    # if "filter" in rss_feed:
                    #     if rss_feed.filter in latest_entry.description:
                    #         post_entry = True
                    # else:
                    #     post_entry = True

                    if post_entry is True:
                        logger.info(
                            "New entry found, posting to Discord channel "
                            f'"{rss_feed.discord_channel.channel}"'
                        )

                        feed_title = rss_feed.name
                        feed_entry_link = latest_entry.link

                        discord_message = f"**{feed_title}**\n{feed_entry_link}"

                        aadiscordbot.tasks.send_channel_message_by_discord_id.delay(
                            rss_feed.discord_channel.channel,
                            discord_message,
                            embed=False,
                        )

                        cache.set(
                            rss_cache_key + "_feed_entry_guid", feed_entry_guid, None
                        )
            except Exception as e:
                logger.info(str(e))
                pass
    else:
        logging.info(
            "AA Discordbot (https://github.com/pvyParts/allianceauth-discordbot) "
            "needs to be installed and configured."
        )
